@@ -1,7 +1,7 @@
 /*
  * KFritz
  *
- * Copyright (C) 2010 Joachim Wilke <kfritz@joachim-wilke.de>
+ * Copyright (C) 2010-2012 Joachim Wilke <kfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,6 +66,8 @@ KFritzWindow::KFritzWindow()
 
 	connect(this, SIGNAL(signalNotification(QString, QString, bool)), this, SLOT(slotNotification(QString, QString, bool)));
 
+	dbusIface = new KFritzDbusService(this);
+
 	initIndicator();
 	updateMissedCallsIndicator();
 	progressIndicator = NULL;
@@ -117,6 +119,8 @@ KFritzWindow::KFritzWindow()
 
 	setupGUI();
 	KXmlGuiWindow::stateChanged("NoEdit");
+	KXmlGuiWindow::stateChanged("NoFB");
+
 
 	// remove handbook menu entry
 	actionCollection()->action("help_contents")->setVisible(false);
@@ -127,6 +131,7 @@ KFritzWindow::~KFritzWindow()
 	// move logging to console
 	fritz::Config::SetupLogging(&std::clog, &std::cout, &std::cerr);
 
+	delete dbusIface;
 	delete libFritzInit;
 }
 
@@ -282,19 +287,18 @@ void KFritzWindow::setupActions() {
 	actionCollection()->addAction("copyNumber", aCopyNumber);
 	connect(aCopyNumber, SIGNAL(triggered(bool)), this, SLOT(copyNumberToClipboard()));
 
-	KAction *aSetDefaultType = new KAction(this);
-	aSetDefaultType->setText(i18n("Set as default"));
-	aSetDefaultType->setIcon(KIcon("favorites"));
-	actionCollection()->addAction("setDefaultType", aSetDefaultType);
-	connect(aSetDefaultType, SIGNAL(triggered(bool)), this, SLOT(setDefault()));
+	KAction *aSetDefault = new KAction(this);
+	aSetDefault->setText(i18n("Set as default"));
+	aSetDefault->setIcon(KIcon("favorites"));
+	actionCollection()->addAction("setDefault", aSetDefault);
+	connect(aSetDefault, SIGNAL(triggered(bool)), this, SLOT(setDefault()));
 
 	KSelectAction *aSetType = new KSelectAction(this);
 	aSetType->setText(i18n("Set type"));
+	// WARNING: mapIndexToType() has to be checked if code changes here:
 	for (size_t p = fritz::FonbookEntry::TYPE_HOME; p < fritz::FonbookEntry::TYPES_COUNT; p++) {
 		aSetType->addAction(KFonbookModel::getTypeName((fritz::FonbookEntry::eType) p));
 	}
-
-	//TODO: icon?
 	actionCollection()->addAction("setType", aSetType);
 	connect(aSetType, SIGNAL(triggered(int)), this, SLOT(setType(int)));
 
@@ -345,17 +349,17 @@ void KFritzWindow::setupActions() {
 	aSeparator->setSeparator(true);
 	actionCollection()->addAction("separator2", aSeparator);
 
+	KStandardAction::save(this, SLOT(save()), actionCollection());
 	KStandardAction::cut(this, SLOT(cutEntry()), actionCollection());
 	KStandardAction::copy(this, SLOT(copyEntry()), actionCollection());
 	KStandardAction::paste(this, SLOT(pasteEntry()), actionCollection());
 	KStandardAction::quit(this, SLOT(quit()), actionCollection());
-
 }
 
 void KFritzWindow::initIndicator() {
-	QIndicate::Server *iServer = NULL;
 	missedCallsIndicator = NULL;
 #ifdef INDICATEQT_FOUND
+	QIndicate::Server *iServer = NULL;
 	iServer = QIndicate::Server::defaultInstance();
 	iServer->setType("message.irc");
 	KService::Ptr service = KService::serviceByDesktopName(appName);
@@ -382,8 +386,8 @@ void KFritzWindow::updateMissedCallsIndicator() {
 	fritz::CallList *callList = fritz::CallList::getCallList(false);
 	if (!callList)
 		return;
-	size_t missedCallCount = callList->MissedCalls(KSettings::lastKnownMissedCall());
 #ifdef INDICATEQT_FOUND
+	size_t missedCallCount = callList->MissedCalls(KSettings::lastKnownMissedCall());
 	missedCallsIndicator->setCountProperty(missedCallCount);
 	if (missedCallCount == 0)
 		missedCallsIndicator->hide();
@@ -451,6 +455,7 @@ void KFritzWindow::updateMainWidgets(bool b)
     if (first.length()) {
     	do {
     		KFonbookModel *modelFonbook = new KFonbookModel(fm->GetTechId());
+    		connect(modelFonbook, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(updateFonbookState()));
 
     		QAdaptTreeView *treeFonbook = new QAdaptTreeView(this);
     		treeFonbook->setAlternatingRowColors(true);
@@ -467,7 +472,7 @@ void KFritzWindow::updateMainWidgets(bool b)
     		treeFonbook->addAction(actionCollection()->action("separator2"));
     		treeFonbook->addAction(actionCollection()->action("dialNumber"));
     		treeFonbook->addAction(actionCollection()->action("copyNumber"));
-    		treeFonbook->addAction(actionCollection()->action("setDefaultType")); //TODO: misleading name 'type'
+    		treeFonbook->addAction(actionCollection()->action("setDefault"));
     		treeFonbook->addAction(actionCollection()->action("setType"));
     		treeFonbook->setContextMenuPolicy(Qt::ActionsContextMenu);
     		connect(treeFonbook->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(updateFonbookContextMenu(const QModelIndex&, const QModelIndex&)));
@@ -489,6 +494,20 @@ void KFritzWindow::updateMainWidgets(bool b)
     connect(treeCallList->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(updateCallListContextMenu(const QModelIndex&, const QModelIndex&)));
 }
 
+void KFritzWindow::save() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+
+	if (container->isFonbook()) {
+		std::string techid = container->getFonbookModel()->getFonbook()->GetTechId();
+		fritz::FonbookManager *fm = fritz::FonbookManager::GetFonbookManager();
+		fritz::Fonbooks *fbs = fm->GetFonbooks();
+		fritz::Fonbook  *fb  = (*fbs)[techid];
+		fb->Save();
+		statusBar()->showMessage(i18n("%1 saved.", toUnicode(fb->GetTitle())), 0);
+		updateFonbookState();
+	}
+}
+
 //TODO: quit using systray
 void KFritzWindow::quit() {
 	// check for pending changes
@@ -563,11 +582,15 @@ void KFritzWindow::setDefault() {
 	}
 }
 
+fritz::FonbookEntry::eType KFritzWindow::mapIndexToType(int index) {
+	return static_cast<fritz::FonbookEntry::eType>(index+1);
+}
+
 void KFritzWindow::setType(int index) {
 	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
 	QAdaptTreeView *treeView = container->getTreeView();
 	if (container->isFonbook()) {
-		container->getFonbookModel()->setType(treeView->currentIndex(), (fritz::FonbookEntry::eType)(index+1)); //TODO: improve mapping index->type
+		container->getFonbookModel()->setType(treeView->currentIndex(), mapIndexToType(index));
 	}
 }
 
@@ -576,16 +599,18 @@ void KFritzWindow::reload() {
 }
 
 void KFritzWindow::reconnectISP() {
-	fritz::FritzClient fc;
-	if (fc.reconnectISP())
+	fritz::FritzClient *fc = fritz::gConfig->fritzClientFactory->create();
+	if (fc->reconnectISP())
 		KMessageBox::information(this, i18n("Reconnect initiated."));
 	else
 		KMessageBox::error(this, i18n("Reconnect failed."));
+	delete fc;
 }
 
 void KFritzWindow::getIP() {
-	fritz::FritzClient fc;
-	std::string ip = fc.getCurrentIP();
+	fritz::FritzClient *fc = fritz::gConfig->fritzClientFactory->create();;
+	std::string ip = fc->getCurrentIP();
+	delete fc;
 	KMessageBox::information(this, i18n("Current IP address is: %1", ip.size() ? ip.c_str() : i18n("unknown")));
 }
 
@@ -679,7 +704,14 @@ void KFritzWindow::updateActionProperties(int tabIndex __attribute__((unused)))
 	if (container->isFonbook()) {
 		if (container->getFonbookModel()->flags(QModelIndex()) & QFlag(Qt::ItemIsEditable))
 			KXmlGuiWindow::stateChanged("WriteableFB");
+		if (container->getFonbookModel()->getFonbook()->isModified())
+			KXmlGuiWindow::stateChanged("DirtyFB");
+		else
+			KXmlGuiWindow::stateChanged("CleanFB");
+	} else {
+		KXmlGuiWindow::stateChanged("NoFB");
 	}
+
 }
 
 void KFritzWindow::updateCallListContextMenu(const QModelIndex &current, const QModelIndex &previous __attribute__((unused))) {
@@ -695,7 +727,7 @@ void KFritzWindow::updateFonbookContextMenu(const QModelIndex &current, const QM
 	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
 	KSelectAction *action = static_cast<KSelectAction *>(actionCollection()->action("setType"));
 	if (container->isFonbook()) {
-		if (current.column() > 0 && current.column() <= fritz::FonbookEntry::MAX_NUMBERS) {
+		if (current.column() > 0 && current.column() <= ((int) fritz::FonbookEntry::MAX_NUMBERS)) {
 			KFonbookModel *model = container->getFonbookModel();
 			const fritz::FonbookEntry *entry = model->retrieveFonbookEntry(current);
 			fritz::FonbookEntry::eType type = entry->GetType(current.column()-1);
@@ -711,6 +743,18 @@ void KFritzWindow::updateFonbookContextMenu(const QModelIndex &current, const QM
 	}
 }
 
+void KFritzWindow::updateFonbookState() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	if (container->isFonbook()) {
+		if (container->getFonbookModel()->getFonbook()->isModified())
+			KXmlGuiWindow::stateChanged("DirtyFB");
+		else
+			KXmlGuiWindow::stateChanged("CleanFB");
+	} else {
+		KXmlGuiWindow::stateChanged("NoFB");
+	}
+}
+
 void KFritzWindow::setProgressIndicator(QString message) {
 	if (!message.length()) {
 		if (progressIndicator) {
