@@ -1,7 +1,7 @@
 /*
  * libfritz++
  *
- * Copyright (C) 2007-2010 Joachim Wilke <libfritz@joachim-wilke.de>
+ * Copyright (C) 2007-2012 Joachim Wilke <libfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 
 #include "CallList.h"
 #include "FonbookManager.h"
+#include "Listener.h"
 #include "Tools.h"
 
 namespace fritz {
@@ -118,6 +119,7 @@ Config::Config( std::string url, std::string password) {
 	mConfig.lastRequestTime = 0;
 	mConfig.logPersonalInfo = false;
 	CharSetConv::DetectCharset();
+	fritzClientFactory = new FritzClientFactory();
 }
 
 Config::~Config() {
