@@ -1,7 +1,7 @@
 /*
  * libfritz++
  *
- * Copyright (C) 2007-2010 Joachim Wilke <libfritz@joachim-wilke.de>
+ * Copyright (C) 2007-2012 Joachim Wilke <libfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,8 @@
 #include <string>
 #include <vector>
 
+#include "FritzClient.h"
+
 #define NAMESPACE "libfritz++"
 #define LOCATOR "[" << NAMESPACE << "/" <<  \
                 std::string(__FILE__, std::string(__FILE__).rfind('/') == std::string::npos ? \
@@ -52,7 +54,8 @@ public:
 	enum eLoginType {
 		UNKNOWN,
 		PASSWORD,
-		SID
+		SID,
+		LUA
 	};
 private:
 	struct sConfig {
@@ -162,6 +165,8 @@ public:
 	void setActiveFonbook( std::string f )            { mConfig.activeFonbook = f; }
 	bool logPersonalInfo( )							  { return mConfig.logPersonalInfo; };
 	virtual ~Config();
+
+	FritzClientFactory *fritzClientFactory;
 };
 
 extern Config* gConfig;
