@@ -1,7 +1,7 @@
 /*
  * libfritz++
  *
- * Copyright (C) 2007-2010 Joachim Wilke <libfritz@joachim-wilke.de>
+ * Copyright (C) 2007-2012 Joachim Wilke <libfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #include <gcrypt.h>
 
 #include "Config.h"
+#include "Tools.h"
 
 #define RETRY_BEGIN                                  \
 	    ost::Thread::setException(ost::Thread::throwException); \
@@ -51,6 +52,7 @@ namespace fritz {
 ost::Mutex* FritzClient::mutex = new ost::Mutex();
 
 FritzClient::FritzClient() {
+	validPassword = false;
 	mutex->enterMutex();
 	// init libgcrypt
 	gcry_check_version (NULL);
@@ -116,60 +118,70 @@ std::string FritzClient::UrlEncode(std::string &s_input) {
 
 bool FritzClient::Login() {
 	// when using SIDs, a new login is only needed if the last request was more than 5 minutes ago
-	if (gConfig->getLoginType() == Config::SID && (time(NULL) - gConfig->getLastRequestTime() < 300)) {
+	if ((gConfig->getLoginType() == Config::SID || gConfig->getLoginType() == Config::LUA) && (time(NULL) - gConfig->getLastRequestTime() < 300)) {
 		return true;
 	}
 
 	// detect type of login once
 	std::string sXml; // sXml is used twice!
-	if (gConfig->getLoginType() == Config::UNKNOWN || gConfig->getLoginType() == Config::SID) {
+	if (gConfig->getLoginType() == Config::UNKNOWN || gConfig->getLoginType() == Config::SID || gConfig->getLoginType() == Config::LUA) {
 		// detect if this Fritz!Box uses SIDs
-		DBG("requesting login_sid.xml from Fritz!Box.");
-		sXml = httpClient->Get(std::stringstream().flush()
-			<< "/cgi-bin/webcm?getpage=../html/login_sid.xml");
-		if (sXml.find("<iswriteaccess>") != std::string::npos)
-			gConfig->setLoginType(Config::SID);
-		else
-			gConfig->setLoginType(Config::PASSWORD);
+		DBG("requesting login_sid.lua from Fritz!Box.");
+		// might return 404 with older fw-versions, our httpClient throws a SockeException for this, catched here
+		try {
+		  sXml = httpClient->Get(std::stringstream().flush()
+			     << "/login_sid.lua?sid=" << gConfig->getSid());
+		} catch (ost::SockException &se) {}
+		if (sXml.find("<Rights") != std::string::npos)
+			gConfig->setLoginType(Config::LUA);
+		else {
+			DBG("requesting login_sid.xml from Fritz!Box.");
+			sXml = httpClient->Get(std::stringstream().flush()
+					<< "/cgi-bin/webcm?getpage=../html/login_sid.xml");
+			if (sXml.find("<iswriteaccess>") != std::string::npos)
+				gConfig->setLoginType(Config::SID);
+			else
+				gConfig->setLoginType(Config::PASSWORD);
+		}
 	}
 
-	if (gConfig->getLoginType() == Config::SID) {
-		DBG("logging into fritz box using SIDs.");
-		if (gConfig->getSid().length() > 0) {
-			// logout, drop old SID (if FB has not already dropped this SID because of a timeout)
-			DBG("dropping old SID");
-			std::string sDummy;
-			sDummy = httpClient->Post(std::stringstream().flush()
-				<< "/cgi-bin/webcm",
-				std::stringstream().flush()
-				<< "sid="
-				<< gConfig->getSid()
-				<< "&security:command/logout=abc");
+	if (gConfig->getLoginType() == Config::SID || gConfig->getLoginType() == Config::LUA) {
+		std::stringstream loginPath;
+//		std::stringstream postdataLogout;
+		if (gConfig->getLoginType() == Config::LUA) {
+			loginPath << "/login_sid.lua";
+//			postdataLogout << "sid=" << gConfig->getSid() << "&logout=abc";
+		} else {
+			loginPath << "/cgi-bin/webcm";
+//			postdataLogout << "sid=" << gConfig->getSid() << "&security:command/logout=abc";
 		}
+//		DBG("logging into fritz box using SIDs.");
+//		if (gConfig->getSid().length() > 0) {
+//			// logout, drop old SID (if FB has not already dropped this SID because of a timeout)
+//			DBG("dropping old SID");
+//			std::string sDummy;
+//			sDummy = httpClient->Post(loginPath, postdataLogout);
+//		}
 		// check if no password is needed (SID is directly available)
-		size_t pwdFlag = sXml.find("<iswriteaccess>");
-		if (pwdFlag == std::string::npos) {
-			ERR("Error - Expected <iswriteacess> not found in login_sid.xml.");
+		size_t sidStart = sXml.find("<SID>");
+		if (sidStart == std::string::npos) {
+			ERR("Error - Expected field <SID> not found in login_sid.xml or login_sid.lua.");
 			return false;
 		}
-		pwdFlag += 15;
-		if (sXml[pwdFlag] == '1') {
-			// extract SID
-			size_t sidStart = sXml.find("<SID>");
-			if (sidStart == std::string::npos) {
-				ERR("Error - Expected <SID> not found in login_sid.xml.");
-				return false;
-			}
-			sidStart += 5;
+		sidStart += 5;
+		std::string sid = sXml.substr(sidStart, 16);
+		if (sid.compare("0000000000000000") != 0) {
 			// save SID
-			gConfig->setSid(sXml.substr(sidStart, 16));
+			DBG("SID is still valid - all ok.");
+			gConfig->setSid(sid);
 			gConfig->updateLastRequestTime();
 			return true;
 		} else {
+			DBG("We need to log in.");
 			// generate response out of challenge and password
 			size_t challengeStart = sXml.find("<Challenge>");
 			if (challengeStart == std::string::npos) {
-				ERR("Error - Expected <Challenge> not found in login_sid.xml.");
+				ERR("Error - Expected <Challenge> not found in login_sid.xml or login_sid.lua.");
 				return false;
 			}
 			challengeStart += 11;
@@ -178,27 +190,36 @@ bool FritzClient::Login() {
             std::string response = CalculateLoginResponse(challenge);
             // send response to box
 			std::string sMsg;
-			sMsg = httpClient->Post(std::stringstream().flush()
-				<< "/cgi-bin/webcm",
-				std::stringstream().flush()
-				<< "login:command/response="
-				<< response
-				<< "&getpage=../html/de/menus/menu2.html");
-			size_t sidStart = sMsg.find("name=\"sid\"");
-			if (sidStart == std::string::npos) {
-				ERR("Error - Expected sid field not found.");
-				return false;
+
+			std::stringstream postdata;
+			if (gConfig->getLoginType() == Config::SID)
+				postdata  << "login:command/response=" << response
+				          << "&getpage=../html/de/menus/menu2.html";
+			else
+				postdata << "username=&response=" << response;
+			DBG("Sending login request...");
+			sMsg = httpClient->Post(loginPath, postdata);
+			size_t sidStart, sidStop;
+			if (gConfig->getLoginType() == Config::SID) {
+				sidStart = sMsg.find("name=\"sid\"");
+				if (sidStart == std::string::npos) {
+					ERR("Error - Expected sid field not found.");
+					return false;
+				}
+				sidStart = sMsg.find("value=\"", sidStart + 10) + 7;
+				sidStop = sMsg.find("\"", sidStart);
+			} else {
+				sidStart = sMsg.find("<SID>");
+				if (sidStart == std::string::npos) {
+					ERR("Error - Expected sid field not found.");
+					return false;
+				}
+				sidStart += 5;
+				sidStop = sMsg.find("</SID>");
 			}
-			sidStart = sMsg.find("value=\"", sidStart + 10) + 7;
-			size_t sidStop = sMsg.find("\"", sidStart);
 			// save SID
 			gConfig->setSid(sMsg.substr(sidStart, sidStop-sidStart));
-			// check if SID is valid
-			bool isValidSid = false;
-			for (size_t pos=0; pos < gConfig->getSid().length(); pos++)
-				if (gConfig->getSid()[pos] != '0')
-					isValidSid = true;
-			if (isValidSid) {
+			if (gConfig->getSid().compare("0000000000000000") != 0) {
 				DBG("login successful.");
 				gConfig->updateLastRequestTime();
 				return true;
@@ -284,8 +305,20 @@ bool FritzClient::InitCall(std::string &number) {
 
 std::string FritzClient::RequestLocationSettings() {
 	std::string msg;
+
 	RETRY_BEGIN {
-		DBG("Looking up Phone Settings...");
+		if (gConfig->getSid().size()) {
+			DBG("Looking up Phone Settings (using lua)...");
+			try {
+				msg = httpClient->Get(std::stringstream().flush()
+					  << "/fon_num/sip_option.lua?sid=" << gConfig->getSid());
+			} catch (ost::SockException &se) {}
+			if (msg.find("<!-- pagename:/fon_num/sip_option.lua-->") != std::string::npos)
+				return msg;
+			DBG("failed.");
+		}
+
+		DBG("Looking up Phone Settings (using webcm)...");
 		msg = httpClient->Get(std::stringstream().flush()
 			<< "/cgi-bin/webcm?getpage=../html/"
 			<<  GetLang()
@@ -299,8 +332,20 @@ std::string FritzClient::RequestLocationSettings() {
 
 std::string FritzClient::RequestSipSettings() {
 	std::string msg;
+
 	RETRY_BEGIN {
-		DBG("Looking up SIP Settings...");
+		if (gConfig->getSid().size()) {
+			DBG("Looking up SIP Settings (using lua)...");
+			try {
+				msg = httpClient->Get(std::stringstream().flush()
+					  << "/fon_num/fon_num_list.lua?sid=" << gConfig->getSid());
+			} catch (ost::SockException &se) {}
+			if (msg.find("<!-- pagename:/fon_num/fon_num_list.lua-->") != std::string::npos)
+				return msg;
+			DBG("failed.");
+		}
+
+		DBG("Looking up SIP Settings (using webcm)...");
 		msg = httpClient->Get(std::stringstream().flush()
 				<< "/cgi-bin/webcm?getpage=../html/"
 				<< GetLang()
@@ -313,10 +358,11 @@ std::string FritzClient::RequestSipSettings() {
 }
 
 std::string FritzClient::RequestCallList () {
-	std::string msg;
+	std::string msg = "";
+	std::string csv = "";
 	RETRY_BEGIN {
 		// now, process call list
-		DBG("sending callList request.");
+		DBG("sending callList update request.");
 		// force an update of the fritz!box csv list and wait until all data is received
 		msg = httpClient->Get(std::stringstream().flush()
 			<< "/cgi-bin/webcm?getpage=../html/"
@@ -325,30 +371,69 @@ std::string FritzClient::RequestCallList () {
 			<<  GetLang()
 			<< "&var:pagename=foncalls&var:menu=fon"
 			<< (gConfig->getSid().size() ? "&sid=" : "") << gConfig->getSid());
+
+		// new method to request call list (FW >= xx.05.50?)
+		try {
+			DBG("sending callList request (using lua)...");
+			csv = httpClient->Get(std::stringstream().flush() << "/fon_num/foncalls_list.lua?"
+			  	                  << "csv=&sid=" << gConfig->getSid());
+			if (csv.find("Typ;Datum;Name;") != std::string::npos) {
+				// we assume utf8 as encoding
+				// the FB sends encoding in the response, however we do not parse it, yet
+				CharSetConv *conv = new CharSetConv("utf8", CharSetConv::SystemCharacterTable());
+				const char *csv_converted = conv->Convert(csv.c_str());
+				csv = csv_converted;
+				delete(conv);
+				return csv;
+			}
+		} catch (ost::SockException e) {}
+
+		// old method, parsing url to csv from page above
+
 		// get the URL of the CSV-File-Export
 		unsigned int urlPos   = msg.find(".csv");
 		unsigned int urlStop  = msg.find('"', urlPos);
 		unsigned int urlStart = msg.rfind('"', urlPos) + 1;
 		std::string csvUrl    = msg.substr(urlStart, urlStop-urlStart);
 		// retrieve csv list
-		msg = "";
-		msg = httpClient->Get(std::stringstream().flush()
-			<< "/cgi-bin/webcm?getpage="
-			<<  csvUrl
-			<< (gConfig->getSid().size() ? "&sid=" : "") << gConfig->getSid());
+		DBG("sending callList request (using webcm)...");
+		csv = httpClient->Get(std::stringstream().flush()
+				<< "/cgi-bin/webcm?getpage="
+				<<  csvUrl
+				<< (gConfig->getSid().size() ? "&sid=" : "") << gConfig->getSid());
+
 		// convert answer to current SystemCodeSet (we assume, Fritz!Box sends its answer in latin15)
 		CharSetConv *conv = new CharSetConv("ISO-8859-15", CharSetConv::SystemCharacterTable());
-		const char *msg_converted = conv->Convert(msg.c_str());
-		msg = msg_converted;
+		const char *csv_converted = conv->Convert(csv.c_str());
+		csv = csv_converted;
 		delete(conv);
 	} RETRY_END
-	return msg;
+	return csv;
 }
 
 std::string FritzClient::RequestFonbook () {
 	std::string msg;
+	// new method, returns an XML
 	RETRY_BEGIN {
-		DBG("sending fonbook request.");
+		if (gConfig->getSid().length()) {
+			ost2::MIMEMultipartForm *mmpf = new ost2::MIMEMultipartForm();
+
+			new ost2::MIMEFormData( mmpf, "sid", gConfig->getSid().c_str());
+			new ost2::MIMEFormData( mmpf, "PhonebookId", "0");
+			new ost2::MIMEFormData( mmpf, "PhonebookExportName", "Telefonbuch");
+			new ost2::MIMEFormData( mmpf, "PhonebookExport", "");
+			DBG("sending fonbook XML request.");
+			try {
+				msg = httpClient->PostMIME(std::stringstream().flush()
+				  	  << "/cgi-bin/firmwarecfg", *mmpf);
+			} catch (ost::SockException &se) {}
+			if (msg.find("<phonebooks>") != std::string::npos) {
+				return msg;
+			}
+		}
+
+	// use old fashioned website (for old FW versions)
+		DBG("sending fonbook HTML request.");
 		msg = httpClient->Get(std::stringstream().flush()
 			<< "/cgi-bin/webcm?getpage=../html/"
 			<< GetLang()
@@ -359,21 +444,6 @@ std::string FritzClient::RequestFonbook () {
 			<< (gConfig->getSid().size() ? "&sid=" : "") << gConfig->getSid());
 	} RETRY_END
 
-	// detect if xml export of fonbook is possible
-	if (msg.find("uiPostExportForm") != std::string::npos) {
-		RETRY_BEGIN {
-			ost2::MIMEMultipartForm *mmpf = new ost2::MIMEMultipartForm();
-
-			new ost2::MIMEFormData( mmpf, "sid", gConfig->getSid().c_str());
-			new ost2::MIMEFormData( mmpf, "PhonebookId", "0");
-			new ost2::MIMEFormData( mmpf, "PhonebookExportName", "Telefonbuch");
-			new ost2::MIMEFormData( mmpf, "PhonebookExport", "");
-
-			msg = httpClient->PostMIME(std::stringstream().flush()
-					<< "/cgi-bin/firmwarecfg", *mmpf);
-		} RETRY_END
-	}
-
 	return msg;
 }
 
