@@ -1,7 +1,7 @@
 /*
  * libfritz++
  *
- * Copyright (C) 2007-2010 Joachim Wilke <libfritz@joachim-wilke.de>
+ * Copyright (C) 2007-2012 Joachim Wilke <libfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,9 +60,9 @@ std::string HttpClient::Result() {
 std::string HttpClient::Get(const std::ostream& url) {
 	urlStream->setAgent("Lynx/2.8.5");
 
-	returnCode = urlStream->get(BuildUrl(url).c_str());
+	ost2::URLStream::Error returnCode = urlStream->get(BuildUrl(url).c_str());
 	if (returnCode != ost2::URLStream::errSuccess)
-		THROW(ost::SockException("Could not connect", ost::Socket::errNotConnected));
+		THROW(ost::SockException("Arbitrary error occured", ost::Socket::errNotConnected));
 	return Result();
 }
 
@@ -74,17 +74,17 @@ std::string HttpClient::Post(const std::ostream &url, const std::ostream &postda
 	params[0] = param0;
 	params[1] = 0;
 
-	returnCode = urlStream->post(BuildUrl(url).c_str(), params);
+	ost2::URLStream::Error returnCode = urlStream->post(BuildUrl(url).c_str(), params);
 	if (returnCode != ost2::URLStream::errSuccess)
-		THROW(ost::SockException("Could not connect", ost::Socket::errNotConnected));
+		THROW(ost::SockException("Arbitrary error occured", ost::Socket::errNotConnected));
 	return Result();
 
 }
 
 std::string HttpClient::PostMIME(const std::ostream &url, ost2::MIMEMultipartForm &form) {
-	returnCode = urlStream->post(BuildUrl(url).c_str(), form);
+	ost2::URLStream::Error returnCode = urlStream->post(BuildUrl(url).c_str(), form);
 	if (returnCode != ost2::URLStream::errSuccess)
-		THROW(ost::SockException("Could not connect", ost::Socket::errNotConnected));
+		THROW(ost::SockException("Arbitrary error occured", ost::Socket::errNotConnected));
 	return Result();
 }
 
