@@ -1,7 +1,7 @@
 /*
  * libfritz++
  *
- * Copyright (C) 2007-2010 Joachim Wilke <libfritz@joachim-wilke.de>
+ * Copyright (C) 2007-2012 Joachim Wilke <libfritz@joachim-wilke.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,25 +30,11 @@
 namespace fritz{
 
 NummerzoekerFonbook::NummerzoekerFonbook()
-{
-	title = I18N_NOOP("nummerzoeker.com");
-	techId = "ZOEK";
-	displayable = false;
-}
+: LookupFonbook(I18N_NOOP("nummerzoeker.com"), "ZOEK")
+{}
 
-NummerzoekerFonbook::~NummerzoekerFonbook()
-{
-}
-
-bool NummerzoekerFonbook::Initialize() {
-	setInitialized(true);
-	return true;
-}
-
-Fonbook::sResolveResult NummerzoekerFonbook::ResolveToName(std::string number) const {
-	Fonbook::sResolveResult result;
-	result.name = number;
-	result.type = FonbookEntry::TYPE_NONE;
+Fonbook::sResolveResult NummerzoekerFonbook::Lookup(std::string number) const {
+	Fonbook::sResolveResult result(number);
 
 	// resolve only NL phone numbers
 	std::string normNumber = Tools::NormalizeNumber(number);
@@ -100,6 +86,7 @@ Fonbook::sResolveResult NummerzoekerFonbook::ResolveToName(std::string number) c
 	delete (conv);
 	INF("resolves to " << (gConfig->logPersonalInfo() ? name.c_str() : HIDDEN));
 	result.name = name;
+	result.successful = true;
 	return result;
 }
 
