@@ -30,125 +30,8 @@
 
 namespace fritz {
 
-const char *Entities[97][2] = {
-	{"&nbsp;",  " "},
-	{"&iexcl;", "¡"},
-	{"&cent;",  "¢"},
-	{"&pound;", "£"},
-	{"&curren;","€"}, //krazy:exclude=spelling
-	{"&yen;",   "¥"},
-	{"&brvbar;","Š"},
-	{"&sect;",  "§"},
-	{"&uml;",   "š"},
-	{"&copy;",  "©"},
-	{"&ordf;",  "ª"},
-	{"&laquo;", "«"},
-	{"&not;",   "¬"},
-	{"&shy;",   "­"},
-	{"&reg;",   "®"},
-	{"&macr;",  "¯"},
-	{"&deg;",   "°"},
-	{"&plusmn;","±"},
-	{"&sup2;",  "²"},
-	{"&sup3;",  "³"},
-	{"&acute;", "Ž"},
-	{"&micro;", "µ"},
-	{"&para;",  "¶"},
-	{"&middot;","·"},
-	{"&cedil;", "ž"},
-	{"&sup1;",  "¹"},
-	{"&ordm;",  "º"},
-	{"&raquo;", "»"},
-	{"&frac14;","Œ"},
-	{"&frac12;","œ"},
-	{"&frac34;","Ÿ"},
-	{"&iquest;","¿"},
-	{"&Agrave;","À"},
-	{"&Aacute;","Á"},
-	{"&Acirc;", "Â"},
-	{"&Atilde;","Ã"},
-	{"&Auml;",  "Ä"},
-	{"&Aring;", "Å"},
-	{"&AElig;", "Æ"},
-	{"&Ccedil;","Ç"},
-	{"&Egrave;","È"},
-	{"&Eacute;","É"},
-	{"&Ecirc;", "Ê"},
-	{"&Euml;",  "Ë"},
-	{"&Igrave;","Ì"},
-	{"&Iacute;","Í"},
-	{"&Icirc;", "Î"},
-	{"&Iuml;",  "Ï"},
-	{"&ETH;",   "Ð"},
-	{"&Ntilde;","Ñ"},
-	{"&Ograve;","Ò"},
-	{"&Oacute;","Ó"},
-	{"&Ocirc;", "Ô"},
-	{"&Otilde;","Õ"},
-	{"&Ouml;",  "Ö"},
-	{"&times;", "×"},
-	{"&Oslash;","Ø"},
-	{"&Ugrave;","Ù"},
-	{"&Uacute;","Ú"},
-	{"&Ucirc;", "Û"},
-	{"&Uuml;",  "Ü"},
-	{"&Yacute;","Ý"},
-	{"&THORN;", "Þ"},
-	{"&szlig;", "ß"},
-	{"&agrave;","à"},
-	{"&aacute;","á"},
-	{"&acirc;", "â"},
-	{"&atilde;","ã"},
-	{"&auml;",  "ä"},
-	{"&aring;", "å"},
-	{"&aelig;", "æ"},
-	{"&ccedil;","ç"},
-	{"&egrave;","è"},
-	{"&eacute;","é"},
-	{"&ecirc;", "ê"},
-	{"&euml;",  "ë"},
-	{"&igrave;","ì"},
-	{"&iacute;","í"},
-	{"&icirc;", "î"},
-	{"&iuml;",  "ï"},
-	{"&eth;",   "ð"},
-	{"&ntilde;","ñ"},
-	{"&ograve;","ò"},
-	{"&oacute;","ó"},
-	{"&ocirc;", "ô"},
-	{"&otilde;","õ"},
-	{"&ouml;",  "ö"},
-	{"&divide;","÷"},
-	{"&oslash;","ø"},
-	{"&ugrave;","ù"},
-	{"&uacute;","ú"},
-	{"&ucirc;", "û"},
-	{"&uuml;",  "ü"},
-	{"&yacute;","ý"},
-	{"&thorn;", "þ"},
-	{"&yuml;",  "ÿ"},
-	{"&amp;",   "&"},
-};
-
-std::string XmlFonbook::convertEntities(std::string s) {
-	if (s.find("&") != std::string::npos) {
-		// convert the entities from UTF-8 to current system character table
-		CharSetConv *conv = new CharSetConv("UTF-8", CharSetConv::SystemCharacterTable());
-
-		for (int i=0; i<97; i++) {
-			std::string::size_type pos = s.find(Entities[i][0]);
-			if (pos != std::string::npos) {
-				s.replace(pos, strlen(Entities[i][0]), conv->Convert(Entities[i][1]));
-				i--; //search for the same entity again
-			}
-		}
-		delete (conv);
-	}
-	return s;
-}
-
-
-XmlFonbook::XmlFonbook() {
+XmlFonbook::XmlFonbook(std::string title, std::string techId, bool writeable)
+: Fonbook(title, techId, writeable) {
 	charset = CharSetConv::SystemCharacterTable() ? CharSetConv::SystemCharacterTable() : "UTF-8";
 }
 
@@ -261,6 +144,9 @@ std::string XmlFonbook::SerializeToXml() {
 				case FonbookEntry::TYPE_WORK:
 					typeName="work";
 					break;
+				default:
+					// should not happen
+					break;
 				}
 				result << "<number type=\"" << typeName << "\" "
 						          "quickdial=\"" << fe->GetQuickdial(numberPos) << "\" "
